@@ -425,22 +425,53 @@ def load_module(module):
     return True
 
 
-def unload_module(module_name):
+def unload_module(module_name, remove_dependent=False):
     """
     Run rmmod to unload module
     Parameters:
     \tmodule_name:       module name
     """
     if not module_name:
-        _print("FAIL: unload_module() - requires module parameter")
+        _print("FAIL: unload_module() - requires module_name parameter")
         return False
-    cmd = "rmmod %s" % module_name
+    cmd = "modprobe -r {}".format(module_name)
+
+    if remove_dependent:
+        dep_modules = get_dependent_modules(module_name)
+        if dep_modules:  # print info only if there are any modules to remove
+            _print("INFO: Removing modules dependent on {}".format(module_name))
+            for module in dep_modules:
+                if not unload_module(module, remove_dependent=remove_dependent):
+                    _print("FAIL: unload_module() - Could not unload dependent modules")
+                    return False
+
     if run(cmd) != 0:
-        _print("FAIL: unload_module() - Could not unload: %s" % module_name)
+        _print("FAIL: unload_module() - Could not unload: {}".format(module_name))
         return False
+
     return True
 
 
+def get_dependent_modules(module_name):
+    """
+    Returns list of modules that loaded this module as a dependency
+    Useful when removing parent modules (error "Module is in use by: ")
+    Parameters:
+    \tmodule_name:      module_name
+    """
+    if not module_name:
+        _print("FAIL: get_dependent_modules() - requires module_name parameter")
+        return None
+    cmd = 'cat /proc/modules | grep -Ew "^{}" | cut -d \' \' -f 4 | tr "," " "'.format(module_name)
+    ret, dependent_modules = run(cmd, return_output=True, verbose=False)
+    if '-' == dependent_modules:
+        return []  # No dependent modules found
+    if ret != 0:
+        _print("FAIL: get_dependent_modules() - failed to get a list of modules")
+        return None
+    return dependent_modules.split()
+
+
 def is_module_loaded(module_name):
     """
     Check if given module is loaded
@@ -999,7 +1030,7 @@ def get_udev_property(device_name, property_key):
     :return property_value: eg. for ID_SERIAL: '360fff19abdd9f5fb943525d45126ca27'
     """
     if not device_name:
-        _print("FAIL: get_udev_property() - requires device_name parameter")
+        _print("WARN: get_udev_property() - requires device_name parameter")
         return None
 
     # Converts for example mpatha to /dev/mapper/mpatha or sda to /dev/sda
@@ -1015,10 +1046,10 @@ def get_udev_property(device_name, property_key):
     ret, property_value = run("udevadm info -q property --name=%s | grep %s= | cut -d = -f 2" % (device, property_key),
                               return_output=True, verbose=False)
     if ret:
-        _print("FAIL: Could not get udevadm info of device '%s'" % device)
+        _print("WARN: Could not get udevadm info of device '%s'" % device)
         return None
     if not property_value:
-        _print("FAIL: Could not find property '%s' in udevadm info of device '%s'" % (property_key, device))
+        _print("WARN: Could not find property '%s' in udevadm info of device '%s'" % (property_key, device))
         return None
 
     return property_value
@@ -1115,7 +1146,7 @@ def get_device_wwid(device):
     """
 
     if device.startswith('vd'):
-        print("WARN: %s: Presuming virtual disk does not have wwid." % device)
+        print("INFO: %s: Presuming virtual disk does not have wwid." % device)
         return None
 
     serial = get_udev_property(device_name=device, property_key='ID_SERIAL')
@@ -1123,7 +1154,7 @@ def get_device_wwid(device):
         dm_uuid = get_udev_property(device_name=device, property_key='DM_UUID')
         serial = dm_uuid.replace('mpath-', '')
     if not serial:
-        _print("FAIL: get_device_wwid() - Could not find WWID for %s" % device)
+        _print("INFO: get_device_wwid() - Could not find WWID for %s" % device)
         return None
 
     return serial
@@ -1174,7 +1205,7 @@ def get_partitions(device):
     Return a list of all parition numbers from the device
     """
     if not device:
-        _print("FAIL: get_partitions() - requires device as parameter")
+        _print("WARN: get_partitions() - requires device as parameter")
         return None
 
     cmd = "parted -s %s print" % device
