@@ -20,6 +20,7 @@ __author__ = "Jakub Krysl"
 __copyright__ = "Copyright (c) 2018 Red Hat, Inc. All rights reserved."
 
 import re  # regex
+import json
 import libsan.host.linux
 from libsan.host.cmdline import run
 from libsan.host.cli_tools import Wrapper
@@ -98,6 +99,21 @@ class Stratis(Wrapper):
             _print(f"WARN: Running command: '{cmd}' failed.")
         return ret
 
+    def get_pool_uuid(self, pool_name):
+        ret, data = self._run(cmd="report", return_output=True)
+        if ret == 0 and data:
+            try:
+                report = json.loads(data)
+            except json.JSONDecodeError:
+                print("FAIL: Could not deserialize data returned from 'stratis report' command!")
+                return None
+            for pool in report["pools"]:
+                if pool_name == pool["name"]:
+                    print(f"INFO: Found UUID: {pool['uuid']} for pool_name: {pool_name}.")
+                    return pool["uuid"]
+            print(f"FAIL: Could not find pool UUID for provided pool_name: {pool_name}!")
+            return None
+
     def pool_create(self, pool_name=None, blockdevs=None, force=False, redundancy=None, key_desc=None,
                     tang_url=None, thumbprint=None, clevis=None, trust_url=None, no_overprovision=None, **kwargs):
         cmd = "pool create "
@@ -125,8 +141,13 @@ class Stratis(Wrapper):
         })
         return self._run(cmd, **self._remove_nones(kwargs))
 
-    def pool_list(self, **kwargs):
-        return self._run("pool list", **kwargs)
+    def pool_list(self, pool_uuid=None, stopped_pools=None, **kwargs):
+        cmd = "pool list "
+        if pool_uuid:
+            cmd += f"--uuid {pool_uuid} "
+        if stopped_pools:
+            cmd += "--stopped "
+        return self._run(cmd, **self._remove_nones(kwargs))
 
     def pool_destroy(self, pool_name=None, **kwargs):
         cmd = "pool destroy "
@@ -256,6 +277,20 @@ class Stratis(Wrapper):
             cmd += f"--uuid {pool_uuid} "
         return self._run(cmd, **self._remove_nones(kwargs))
 
+    def pool_start(self, pool_uuid=None, unlock_method=None, **kwargs):
+        cmd = "pool start "
+        if unlock_method:
+            cmd += f"--unlock-method {unlock_method} "
+        if pool_uuid:
+            cmd += f"{pool_uuid} "
+        return self._run(cmd, **self._remove_nones(kwargs))
+
+    def pool_stop(self, pool_name=None, **kwargs):
+        cmd = "pool stop "
+        if pool_name:
+            cmd += f"{pool_name} "
+        return self._run(cmd, **self._remove_nones(kwargs))
+
     def blockdev_list(self, pool_name=None, **kwargs):
         cmd = "blockdev list "
         if pool_name:
@@ -353,6 +388,7 @@ class Stratis(Wrapper):
             cmd += f"{key_desc} "
         return self._run(cmd, **kwargs)
 
+    # Pool unlock has been removed in favor of pool_start in Stratisd 3.2.0
     def pool_unlock(self, **kwargs):
         return self._run("pool unlock", **kwargs)
 
@@ -362,5 +398,8 @@ class Stratis(Wrapper):
     def daemon_version(self, **kwargs):
         return self._run("daemon version", **kwargs)
 
+    def debug_refresh(self, **kwargs):
+        return self._run("debug refresh", **kwargs)
+
     def version(self, **kwargs):
         return self._run("--version", **kwargs)
